@@ -1,4 +1,5 @@
 
+from io import TextIOWrapper
 import json
 import os
 import tempfile
@@ -11,14 +12,27 @@ from server_sender import ServerSender
 
 JSON_PATH = 'ray_canvas.json'
 
+def _get_version_tuple_json_dict(json_contents: dict) -> tuple[int, int, int]:
+    if 'version' in json_contents.keys():
+        version_str = json_contents['version']
+        try:
+            version_list = [int(v) for v in version_str.split('.')]
+        except:
+            version_list = [int(v) for v in ray.VERSION.split('.')]
+    else:
+        version_list = [0, 12, 0]
+    
+    return tuple(version_list)
+
+
 class CanvasSaver(ServerSender):
     def __init__(self, session):
         ServerSender.__init__(self)
         self.session = session
 
-        self.group_positions_session = []
-        self.group_positions_config = []
-        self.portgroups = []
+        self.group_positions_session = list[ray.GroupPosition]()
+        self.group_positions_config = list[ray.GroupPosition]()
+        self.portgroups = list[ray.PortGroupMemory]()
         self._config_json_path = "%s/%s" % (
             dirname(RS.settings.fileName()), JSON_PATH)
 
@@ -27,23 +41,38 @@ class CanvasSaver(ServerSender):
 
         with open(self._config_json_path, 'r') as f:
             json_contents = {}
-            gpos_list = []
-            pg_list = []
+            gpos_list = list[ray.GroupPosition]()
+            pg_list = list[ray.PortGroupMemory]()
 
             try:
                 json_contents = json.load(f)
             except json.JSONDecodeError:
                 Terminal.message("Failed to load patchcanvas config file %s" % f)
 
-            if type(json_contents) == dict:
+            # Old group_position port_types_view norm was
+            # full_view = AUDIO | MIDI (1|2 = 3)
+            # now this is:
+            # full_view = AUDIO | MIDI | CV | VIDEO (1|2|4|8 = 15)
+            # (even if VIDEO is not implemented in patchbay yet)
+            
+            # so we need to consider that a full view is a full view
+            # and convert in old sessions port_types_view = 3 -> 15 
+            needs_port_types_view_convert = False
+
+            if isinstance(json_contents, dict):
                 if 'group_positions' in json_contents.keys():
                     gpos_list = json_contents['group_positions']
                 if 'portgroups' in json_contents.keys():
                     pg_list = json_contents['portgroups']
+                    
+                if _get_version_tuple_json_dict(json_contents) < (0, 13, 0):
+                    needs_port_types_view_convert = True
 
             for gpos_dict in gpos_list:
                 gpos = ray.GroupPosition()
                 gpos.write_from_dict(gpos_dict)
+                if needs_port_types_view_convert and gpos.port_types_view == 3:
+                    gpos.port_types_view = 15
                 self.group_positions_config.append(gpos)
 
             for pg_dict in pg_list:
@@ -78,7 +107,7 @@ class CanvasSaver(ServerSender):
                 distant_guis.append(gui_addr)
 
         if local_guis:
-            session_gpos_dict = {'group_positions': []}
+            session_gpos_dict = {'group_positions': list[dict]()}
             for gpos in self.group_positions_session:
                 session_gpos_dict['group_positions'].append(gpos.to_dict())
 
@@ -163,6 +192,7 @@ class CanvasSaver(ServerSender):
         gp = ray.GroupPosition.new_from(*args)
         for group_positions in (self.group_positions_session,
                                 self.group_positions_config):
+            
             for gpos in group_positions:
                 if gpos.is_same(gp):
                     gpos.update(*args)
@@ -180,20 +210,26 @@ class CanvasSaver(ServerSender):
 
         with open(session_canvas_file, 'r') as f:
             json_contents = {}
-            gpos_list = []
+            gpos_list = list[dict]()
 
             try:
                 json_contents = json.load(f)
             except json.JSONDecodeError:
                 Terminal.message("Failed to load session canvas file %s" % f)
 
-            if (type(json_contents) == dict
-                    and 'group_positions' in json_contents.keys()):
-                gpos_list = json_contents['group_positions']
+            needs_port_types_view_convert = False
+
+            if isinstance(json_contents, dict):
+                if 'group_positions' in json_contents.keys():
+                    gpos_list : list[dict]() = json_contents['group_positions']
+                if _get_version_tuple_json_dict(json_contents) < (0, 13, 0):
+                    needs_port_types_view_convert = True
 
             for gpos_dict in gpos_list:
                 gpos = ray.GroupPosition()
                 gpos.write_from_dict(gpos_dict)
+                if needs_port_types_view_convert and gpos.port_types_view == 3:
+                    gpos.port_types_view = 15
                 self.group_positions_session.append(gpos)
 
     def save_json_session_canvas(self, session_path: str):
@@ -205,6 +241,7 @@ class CanvasSaver(ServerSender):
         json_contents = {}
         json_contents['group_positions'] = [
             gpos.to_dict() for gpos in self.group_positions_session]
+        json_contents['version'] = ray.VERSION
 
         with open(session_json_path, 'w+') as f:
             json.dump(json_contents, f, indent=2)
@@ -218,6 +255,7 @@ class CanvasSaver(ServerSender):
             gpos.to_dict() for gpos in self.group_positions_config]
         json_contents['portgroups'] = [
             portgroup.to_dict() for portgroup in self.portgroups]
+        json_contents['version'] = ray.VERSION
 
         with open(self._config_json_path, 'w+') as f:
             json.dump(json_contents, f, indent=2)
